@@ -1,16 +1,38 @@
 from __future__ import annotations
 
+import logging
 import math
+import shlex
 from io import StringIO, TextIOWrapper
+
+import numpy as np
+from matplotlib import pyplot as plt
 from serenipy import sqt, dtaselectfilter
 import pandas as pd
 from Bio.SeqUtils.ProtParam import ProteinAnalysis
+from sklearn.preprocessing import MinMaxScaler
+from typing.io import IO
+
 from .util import calculate_protein_coverage, get_unmodified_peptide, map_protein_to_peptides, map_peptide_to_specid
 
 
-def convert_to_csv(sqt_content: TextIOWrapper | StringIO, filename=None) -> pd.DataFrame:
+def parse_dta_args(arg_string):
+    args = shlex.split(arg_string)
+    arg_dict = {}
+    for i in range(len(args)):
+        if args[i].startswith("-"):
+            if i + 1 < len(args) and not args[i + 1].startswith("-"):
+                arg_dict[args[i]] = args[i + 1]
+            else:
+                arg_dict[args[i]] = True
+    return arg_dict
+
+
+def convert_to_csv(sqt_content: TextIOWrapper | StringIO, filename=None, xcorr_filter=None, max_mline=None) -> pd.DataFrame:
     """
     Convert SQT filee into a pandas df
+    :param max_mline:
+    :param xcorr_filter:
     :param sqt_content: contents of the sqt file
     :param filename: name of the sqt file
     :return: the pandas Dataframe
@@ -18,25 +40,28 @@ def convert_to_csv(sqt_content: TextIOWrapper | StringIO, filename=None) -> pd.D
 
     _, _, s_lines = sqt.from_sqt(sqt_content)
 
-    data = {'low_scan': [], 'high_scan': [], 'charge': [], 'process_time': [], 'server': [],
+    data = {'low_scan': [], 'high_scan': [], 'charge': [],
             'experimental_mass': [], 'total_ion_intensity': [], 'lowest_sp': [], 'number_matches': [],
             'experimental_ook0': [], 'experimental_mz': [], 'corrected_ook0': [],
 
             'xcorr_rank': [], 'sp_rank': [], 'calculated_mass': [], 'delta_cn': [], 'xcorr': [], 'sp': [],
             'matched_ions': [],
             'expected_ions': [], 'sequence': [], 'validation_status': [], 'predicted_ook0': [], 'tims_score': [],
-            'tims_b_score_m2': [],
-            'tims_b_score_best_m': [],
 
             'locuses': [], 'target': [], 'm_line': []}
 
     for s_line in s_lines:
         for i, m_line in enumerate(s_line.m_lines[:-1]):
+
+            if max_mline is not None and i >= max_mline:
+                break
+
+            if xcorr_filter is not None and m_line.xcorr <= xcorr_filter:
+                continue
+
             data['low_scan'].append(s_line.low_scan)
             data['high_scan'].append(s_line.high_scan)
             data['charge'].append(s_line.charge)
-            data['process_time'].append(s_line.process_time)
-            data['server'].append(s_line.server)
             data['experimental_mass'].append(s_line.experimental_mass)
             data['total_ion_intensity'].append(s_line.total_ion_intensity)
             data['lowest_sp'].append(s_line.lowest_sp)
@@ -57,8 +82,6 @@ def convert_to_csv(sqt_content: TextIOWrapper | StringIO, filename=None) -> pd.D
             data['validation_status'].append(m_line.validation_status)
             data['predicted_ook0'].append(m_line.predicted_ook0)
             data['tims_score'].append(m_line.tims_score)
-            data['tims_b_score_m2'].append(m_line.tims_b_score_m2)
-            data['tims_b_score_best_m'].append(m_line.tims_b_score_best_m)
 
             data['locuses'].append(' '.join([l_line.locus_name for l_line in m_line.l_lines]))
 
@@ -69,9 +92,58 @@ def convert_to_csv(sqt_content: TextIOWrapper | StringIO, filename=None) -> pd.D
     df.index.name = 'id'
     if filename:
         df['file'] = filename
+
     return df
 
 
+def plot_alignment(rts, expt_masses, shifts, calc_masses):
+    fig, ax = plt.subplots()
+    ax.scatter(rts, (expt_masses - calc_masses) / calc_masses * 1_000_000, s=0.1, label='Raw')
+    ax.scatter(rts, (expt_masses - shifts - calc_masses) / calc_masses * 1_000_000, s=0.1, label='Aligned')
+
+    ppm_shifts = shifts/calc_masses*1_000_000
+    rts, ppm_shifts = zip(*sorted(zip(rts, ppm_shifts), key=lambda x: x[0]))
+    plt.plot(rts, shifts/calc_masses*1_000_000, c='r', label='lobf')
+    plt.title('Mass Alignment')
+    plt.xlabel('Normalized Retention Time')
+    plt.ylabel('PPM')
+    ax.legend()
+    return fig
+
+
+def align_mass(df, mass_alignment_dim, mass_alignment_percentile):
+    scaler = MinMaxScaler()
+    rts = scaler.fit_transform(df['low_scan'].values.reshape(-1, 1)).reshape(-1)
+    x_corr_perc = np.percentile(df['xcorr'], mass_alignment_percentile)
+    align_index = df['xcorr'] >= x_corr_perc
+    align_rts = rts[align_index]
+    align_mass_ppm_diff = (1_000_000 * (df['experimental_mass'] - df['calculated_mass']) / df['calculated_mass'])[
+        align_index]
+
+    try:
+        alignment_func = np.poly1d(np.polyfit(align_rts, align_mass_ppm_diff, mass_alignment_dim))
+    except np.linalg.LinAlgError as err:
+        logging.error(f'Error aligning masses: {err}')
+        return None
+
+    shifts = alignment_func(rts) * df['experimental_mass'] / 1_000_000
+    fig = plot_alignment(rts[align_index], df['experimental_mass'][align_index], shifts[align_index], df['calculated_mass'][align_index])
+    df['experimental_mass'] = df['experimental_mass'] - shifts
+    return fig
+
+
+def align_mobility(df):
+    scaler = MinMaxScaler()
+    rts = scaler.fit_transform(df['low_scan'].values.reshape(-1, 1)).reshape(-1)
+    x_corr_perc = np.percentile(df['xcorr'], 95)
+    align_index = df['xcorr'] >= x_corr_perc
+    align_rts = rts[align_index]
+    align_mobility_diff = (df['experimental_ook0'] - df['predicted_ook0'])[align_index]
+    alignment_func = np.poly1d(np.polyfit(align_rts, align_mobility_diff, 1))
+    shifts = alignment_func(rts)
+    df['experimental_ook0'] = df['experimental_ook0'] - shifts
+
+
 def convert_to_moka(sqt_df: pd.DataFrame):
     """
     Converts a sqt df to a mokapot input df (pin format)
@@ -87,13 +159,11 @@ def convert_to_moka(sqt_df: pd.DataFrame):
     perc_df.loc[perc_df['Label'] == 1, 'Label'] = 1
 
     perc_df['ScanNr'] = sqt_df['low_scan']
-    perc_df['ExpMass'] = sqt_df['experimental_mass']
-    perc_df['CalcMass'] = sqt_df['calculated_mass']
-
-    # perc_df['group'] = df['file']
+    perc_df['ExpMass'] = sqt_df['experimental_mass'].replace(0, np.nan)
+    perc_df['CalcMass'] = sqt_df['calculated_mass'].replace(0, np.nan)
 
     perc_df['abs_ppm'] = \
-        abs((sqt_df['experimental_mass'] - sqt_df['calculated_mass']) / sqt_df['calculated_mass'] * 1_000_000)
+        abs((sqt_df['experimental_mass'] - sqt_df['calculated_mass']).divide(sqt_df['calculated_mass']) * 1_000_000)
     perc_df['abs_mass_diff'] = abs(sqt_df['experimental_mass'] - sqt_df['calculated_mass'])
 
     charges = pd.get_dummies(sqt_df['charge'], prefix='charge')
@@ -103,14 +173,19 @@ def convert_to_moka(sqt_df: pd.DataFrame):
     perc_df['xcorr'] = sqt_df['xcorr']
     perc_df['matched_ions'] = sqt_df['matched_ions']
     perc_df['expected_ions'] = sqt_df['expected_ions']
-    perc_df['matched_ion_fraction'] = sqt_df['matched_ions'] / sqt_df['expected_ions']
+    perc_df['matched_ion_fraction'] = sqt_df['matched_ions'].divide(sqt_df['expected_ions'])
+    perc_df['matched_ion_fraction'].replace([np.inf, -np.inf], np.nan, inplace=True)
+
+    if perc_df['matched_ion_fraction'].isnull().values.any():
+        print(
+            f"Detected {perc_df['matched_ion_fraction'].isnull().sum()} nan values in column: matched_ion_fraction")
+        perc_df['matched_ion_fraction'] = perc_df['matched_ion_fraction'].fillna(0)
 
     perc_df['sp'] = sqt_df['sp']
     perc_df['sequence_length'] = [len(get_unmodified_peptide(peptide)) for peptide in sqt_df['sequence']]
 
     perc_df['xcorr_rank'] = sqt_df['xcorr_rank']
     perc_df['sp_rank'] = sqt_df['sp_rank']
-    perc_df['tryp'] = [int(peptide[0] in ['KR']) + int(peptide[-3] in ['KR']) for peptide in sqt_df['sequence']]
 
     m_lines = pd.get_dummies(sqt_df['m_line'], prefix='m_line')
     perc_df = pd.concat([perc_df, m_lines], axis=1)
@@ -133,6 +208,7 @@ def get_filter_results_moka(sqt_df: pd.DataFrame, psm_results: pd.DataFrame, pep
     :param fasta_dict: locus to protein sequence map
     :return: list of DTAFilterResult results
     """
+    spec_id_to_qvalue = {spec_id:q_value for spec_id, q_value in psm_results[['SpecId', 'mokapot q-value']].values}
     peptide_to_specid = map_peptide_to_specid(psm_results)
     protein_to_peptides = map_protein_to_peptides(protein_results, peptide_results)
 
@@ -140,11 +216,11 @@ def get_filter_results_moka(sqt_df: pd.DataFrame, psm_results: pd.DataFrame, pep
     for protein, peptides in protein_to_peptides.items():
         protein_lines = []
         peptide_lines = []
-        peptides = [peptide for peptide in peptides if peptide in peptide_to_specid]
+        peptides = [str(peptide) for peptide in peptides if peptide in peptide_to_specid]
         psm_ids = [psm_id for peptide in peptides for psm_id in peptide_to_specid[peptide]]
         sqt_psm_df = sqt_df.loc[psm_ids]
         for locus in [protein]:
-            protein_sequence = fasta_dict[locus]['sequence']
+            protein_sequence = str(fasta_dict[locus]['sequence'])
             description_name = fasta_dict[locus]['description']
             sequence_count = len({(sequence, charge) for sequence, charge in sqt_psm_df[['sequence', 'charge']].values})
             spectrum_count = len(psm_ids)
@@ -176,7 +252,7 @@ def get_filter_results_moka(sqt_df: pd.DataFrame, psm_results: pd.DataFrame, pep
                                                        m_redundancy=None)
             protein_lines.append(protein_line)
 
-        for _, row in sqt_psm_df.iterrows():
+        for spec_id, row in sqt_psm_df.iterrows():
             peptide = ProteinAnalysis(row['sequence'])
 
             file_path = row['file']
@@ -185,18 +261,29 @@ def get_filter_results_moka(sqt_df: pd.DataFrame, psm_results: pd.DataFrame, pep
             charge = row['charge']
 
             file_name = f'{file_path}.{low_scan}.{high_scan}.{charge}'
+
+            try:
+                ppm = (row['experimental_mass'] - row['calculated_mass']) / row['calculated_mass'] * 1_000_000
+            except ZeroDivisionError:
+                ppm = None
+
+            try:
+                ion_proportion = row['matched_ions'] / row['expected_ions'] * 100
+            except ZeroDivisionError:
+                ion_proportion = None
+
             peptide_line = dtaselectfilter.PeptideLine(
                 unique=None,
                 file_name=file_name,
                 x_corr=row['xcorr'],
                 delta_cn=row['delta_cn'],
-                conf=None,
+                conf=1 - spec_id_to_qvalue[spec_id],
                 mass_plus_hydrogen=row['experimental_mass'],
                 calc_mass_plus_hydrogen=row['calculated_mass'],
-                ppm=(row['experimental_mass'] - row['calculated_mass']) / row['calculated_mass'] * 1_000_000,
+                ppm=ppm,
                 total_intensity=row['total_ion_intensity'],
                 spr=row['sp_rank'],
-                ion_proportion=row['matched_ions'] / row['expected_ions'] * 100,
+                ion_proportion=ion_proportion,
                 redundancy=None,  # add in later
                 sequence=row['sequence'],
                 prob_score=row['sp'],
